#!/usr/bin/env python
"""
This is the config for ctznOSX
"""
import ConfigParser, inspect
from os.path import dirname,abspath,join,isfile
from sys import argv,exit

# ctznOSX Configuration
def ctznConfig( config_file, ctznosxpath ):

    # Default Configuration
    config = {
        'main': {
            'debug': False, 
            'datastore': join(ctznosxpath,'ctznosx.db'),
            'logstore': join(ctznosxpath,'logs'),
            'monitorstore': join(ctznosxpath, "monitors"),
            'reportstore': join(ctznosxpath, "reports")
            }, 
        'reporting': {
            'type': 'http', 
            'enabled': False, # True or False 
            'target': ''
            }, 
        'watcher': {
            'enabled': 'true',
            'timeout': 3, # Powers of
            'send_every': 60, 
            }
        }

    if isfile(config_file):
        cp = ConfigParser.ConfigParser()
        cp.read(config_file)
        for section in cp.sections():
            for option in cp.options(section):
                config[section][option] = cp.get(section, option)
    else:
        print '!! NOTICE - Using default configuration'

    return config



from __future__ import unicode_literals

import argparse
import locale
import sys
from os import listdir,walk,path,environ
from ctznosx import __version__, monitors as Monitors, devicemgmt as Manager
from ctznosx import report as Report
from ctznosx.exceptions import Error
from ctznosx.usage import *
from ctznosx import launcher
from config import ctznConfig

# Get ctznOSX Env and Config
CTZNOSX_PATH = (environ.get('CTZNOSX_PATH') or '/var/lib/ctznosx/')
CTZNOSX_CONFIG = path.join('/etc/', 'ctznosx.conf')

# Config
CONFIG = ctznConfig( CTZNOSX_CONFIG, CTZNOSX_PATH )

def argument_parser(*args, **kwargs):
    format_usage = kwargs.pop('format_usage', None)
    format_help = kwargs.pop('format_help', None)

    class ArgumentParser(argparse.ArgumentParser):

        def format_usage(self):
            return '%s\n' % format_usage

        def format_help(self):
            return '%s\n%s\n' % (self.format_usage(), format_help)

    kwargs['add_help'] = False

    p = ArgumentParser(*args, **kwargs)
    p.add_argument('--help', action='help')
    p.add_argument('-h', action='help')

    return p

# Handles monitor sub-commands
def monitor(args):
    if args is None:
        print MONITOR_USAGE
        sys.exit(1)
    else:
        if args[1] in ('list', 'install', 'remove', 'upgrade'):
            getattr(Monitors, args[1])(args[2:])
        

# Handles monitor sub-commands
def manager(args):
    if args is None:
        print MANAGER_USAGE
        sys.exit(1)        
    else:
        if args[1] in ('register'):
            Manager.register_device()      
        

# Handles clean sub-commands
def clean(args):
    print args

def main(argv=None):
    """
    Handle command line arguments.
    """

    if argv is None:
        argv = sys.argv[1:]
        encoding = locale.getdefaultlocale()[1]
        if encoding:
            argv = [a.decode(encoding) for a in sys.argv[1:]]

    # find command position
    pos, skip_next = 0, False
    for i, arg in enumerate(argv):
        if skip_next:
            skip_next = False
            continue
        else:
            pos = i
            break

    parser = argument_parser(prog='ctznosx', format_usage=USAGE, format_help=HELP)
    parser.add_argument('--verbose', action='store_true')
    parser.add_argument('--dry-run', action='store_true')
    parser.add_argument('--version', action='version', version=__version__)
    parser.add_argument('-v', action='version', version=__version__)
    parser.add_argument('command')

    try:
        # only parse up until command
        args = parser.parse_args(argv[:pos + 1])
        
        # If this is a plugin install/remove
        if args.command in ('monitor'):
            monitor(argv)

        # If this is a manager based command
        elif args.command in ('manager'):
            manager(argv)

        # Clean subcommand
        elif args.command in ('clean'):
            clean(args)

        # Report subcommand
        elif args.command in ('report'):
            Report.run(argv)

        # Generic Run Script
        else:
            launcher.run()

    except Error as error:
        message = '%s' % error
        if not message.endswith('\n'):
            message += '\n'
        parser.exit(1, message)


from __future__ import unicode_literals

import datetime
from os import environ
from sys import exit
from os.path import join
from ctznosx import __version__ as version
from titantools import system as s
import urllib2, urllib, httplib, json
from config import ctznConfig

# Get ctznOSX Env and Config
CTZNOSX_PATH = (environ.get('CTZNOSX_PATH') or '/var/lib/ctznosx/')
CTZNOSX_CONFIG = join('/etc/', 'ctznosx.conf')

# Config
CONFIG = ctznConfig( CTZNOSX_CONFIG, CTZNOSX_PATH )

# Get Defaults
software = s.sw_details()
hardware = s.hw_details()

# Log Prefix
PREFIX = "[ Manager::Register ] "

""" register """
def register_device():

    print PREFIX, "Attempting to register device with remote server"
    data = {}

    data["serial"] = hardware["serial_number"]
    data["uuid"] = hardware["hardware_uuid"]
    data["make"] = hardware["machine_make"]
    data["model"] = hardware["model_short"]
    data["cpu_type"] = hardware["cpu_type"]
    data["cpu_speed"] = hardware["cpu_speed"]
    data["physical_memory"] = hardware["physical_memory"]
    data["os_version"] = software["os_version"]
    data["os_build"] = software["os_build"]

    code, resp = send_request("%s/connect/%s" % (CONFIG["reporting"]["target"], data["serial"]), data)
    
    if code == 0:
        print PREFIX, "Unable To Communicate With Registration Server"
        return False
    
    elif code == 304:
        print PREFIX, "Device already registered"
        return True

    elif code == 200:
        print PREFIX, "Device Registered Successfully"
        return True

    else:
        print PREFIX, "Registration Failed, Error: %d - '%s'" % (code, response)
        return False


def send_request( target, data):
    try:
        request = urllib2.Request(target, urllib.urlencode(data) )
        request.add_header("User-Agent", "ctznOSX %s" % version)
        opener = urllib2.build_opener()
        response = opener.open(request)
        response_object = response.getcode(), response

    except urllib2.HTTPError, e:
        if e.code == 307:
            print PREFIX, "This device needs to be registered"
            for line in str(e.headers).splitlines():
                if "Location" in line:
                    new_target = line.split(": ", 1)[1]
                    response_object = send_request( new_target, data )
        else:
            response_object = e.code, e.read()

    except urllib2.URLError, e:
        response_object = 0, 'Connection Refused'


    return response_object



from __future__ import unicode_literals

def _(text):
    return text.strip('\n')

HELP = _("""
  -h, --help         show help message
  -v, --version      show version
  --verbose          show script while running
  --dry-run          show script without running

commands:
  run                invoke the scanner
  device             manage device
  monitor            manage monitors
  report             display report

""")

USAGE = _("""
                                  MMP\"""\""YMM MP\"""\"""`MM M""MMMM""M
           dP                     M' .mmm. `M M  mmmmm..M M  `MM'  M
           88                     M  MMMMM  M M.      `YM MM.    .MM
.d8888b. d8888P d888888b 88d888b. M  MMMMM  M MMMMMMM.  M M  .mm.  M
88'  `""   88      .d8P' 88'  `88 M. `MMM' .M M. .MMM'  M M  MMMM  M
88.  ...   88    .Y8P    88    88 MMb     dMM Mb.     .dM M  MMMM  M
`88888P'   dP   d888888P dP    dP MMMMMMMMMMM MMMMMMMMMMM MMMMMMMMMM

Usage: ctznosx [--config=CONFIG] command

%s""" % HELP)

# Unused ATM
#  clean              clean data store contents

MONITOR_USAGE = _("""
Usage: ctznosx monitor [ install | list | upgrade | remove ]

commands:
  list               show all installed monitors
  install            installs specified monitor
  upgrade            upgrades specified monitor
  remove             removes specified monitor
""")


MANAGER_USAGE = _("""
Usage: ctznosx manager [ register ]

commands:
  register           register this device with a remote server
""")



#!/usr/bin/env python
#
# Todo: Read config file, use target as test for urlopen2
# Todo: Check to make sure we don't duplicate queue files
#

import sys
import logging
import hashlib
from ctznosx import __version__ as version
from sys import exit
import urllib2, urllib, httplib, json
from os import path,walk,remove,environ
from os.path import dirname, realpath, isfile, join, splitext, basename
from binascii import b2a_hex,hexlify
from config import ctznConfig
from titantools.orm import TiORM as ctznORM
from titantools.system import shell_out,hw_serial as get_device_serial
from zlib import compress
from time import sleep,strftime,strptime,gmtime,mktime

# Get ctznOSX Env and Config
CTZNOSX_PATH = (environ.get('CTZNOSX_PATH') or '/var/lib/ctznosx/')
CTZNOSX_CONFIG = join('/etc/', 'ctznosx.conf')

# Config
CONFIG = ctznConfig( CTZNOSX_CONFIG, CTZNOSX_PATH )

# Set datastore
DATASTORE = CONFIG['main']['datastore']

# Load ORM 
ORM = ctznORM(DATASTORE)

# Set device id
DEVICEID = get_device_serial()

# Create Run Prefix
RUN_PREFIX = "[%s] " % strftime("%a, %d %b %Y %H:%M:%S-%Z", gmtime())

# Load default schema
ORM.initialize_table('watcher', {u'date': {u'nullable': False, u'type': u'text'}, 
                u'utime': {u'type': u'integer'}, 
                u'module': {u'type': u'text'}, 
                u'status': {u'type': u'integer'}})

# Get reporting target
REPORTING_TARGET = "%s%s" % (CONFIG['reporting']['target'], "observer")

if REPORTING_TARGET is None or REPORTING_TARGET == "":
  print "[!] No place to report to"
  exit()


# Check if Watcher is enabled
if CONFIG['watcher']['enabled'] is "false":
  print "[!] Watcher is disabled"
  exit()

# Configure Logging
logging.basicConfig(format='%(message)s', level=logging.INFO)

def generate_reports():

  # Report Timestamp
  unix_time = int(mktime(gmtime()))
  exec_time = strftime("%a, %d %b %Y %H:%M:%S-%Z", gmtime())

  # Query all tables
  all_tables = ORM.select('sqlite_master', 'name',  "type = 'table' and name != 'watcher'")

  # How many wins do we need
  passes_needed = 0
  passes_had = 0

  # Loop through tables
  for table in [table for table in all_tables]:
    # Last success
    last_success = ORM.raw_sql('SELECT * FROM watcher WHERE module="%s" ORDER BY date DESC LIMIT 1' % (table['name']))

    # Get table data
    if len(last_success[0]) is 0:
      # Logging is cool
      temp_utime = unix_time
      ORM.raw_sql("INSERT INTO watcher (date,status,utime,module) VALUES ('%s', 0,'%s', '%s')" % (exec_time, temp_utime, table['name']))
      logging.info("%sCollecting data for [%s] since ever" % (RUN_PREFIX,table['name']))
      results = ORM.select(table['name'], '*')
    else:
      # Logging is cool
      temp_utime = int(last_success[0][4])
      logging.info("%sCollecting data for [%s] since %s" % (RUN_PREFIX,table['name'],last_success[0][1]))
      results = ORM.select(table['name'], '*', 'unixtime > %d' % temp_utime)

    if results is None or len(results) == 0:
      ORM.raw_sql("UPDATE watcher SET status=0, utime = '%d' WHERE module = '%s'" % (unix_time, table['name']))
      continue

    # Dump table to json
    table_json = json.dumps({"module": table['name'], "data": results})

    # Compress the data
    compressed = compress(table_json)

    # Digest
    content_digest = hashlib.sha256(compressed).hexdigest()

    # Send the table data upstream
    target = "%s/%s" % (REPORTING_TARGET, DEVICEID)
    logging.info("\tSending request to '%s'" % target)
    code, response = send_request(target, {'serial': DEVICEID, 'digest': content_digest, 'stream': compressed})
    try:
      logging.info("\tResponse: [%d] @ '%s'" % (code, (response.read())))
    except:
      logging.info("\tResponse: [%d] @ '%s'" % (code, (response)))


    if code == 202:
      passes_had += 1
      ORM.raw_sql("UPDATE watcher SET status=1, utime = %d WHERE module = '%s'" % (unix_time, table['name']))
    else:
      ORM.raw_sql("UPDATE watcher SET status=0, utime = %d WHERE module = '%s'" % (temp_utime, table['name']))

  exit()


# Send the request
def send_request( target, data):

  try:
    request = urllib2.Request(target, urllib.urlencode(data) )
    request.add_header("User-Agent", "ctznOSX %s" % version)
    opener = urllib2.build_opener()
    response = opener.open(request)
    response_object = response.getcode(), response

  except urllib2.HTTPError, e:
    response_object = e.code, e.read()

  except urllib2.URLError, e:
    response_object = 0, 'Connection Refused'

  return response_object


# Run the meat and potatoes
def run():

  # Check if internet is available
  try_count = 0

  while True:

    try:
      target = "%s/%s" % (REPORTING_TARGET, DEVICEID)
      logging.info("%sChecking connectivity to: '%s'" % (RUN_PREFIX,target))
      code, response = send_request(target, {'serial': DEVICEID, 'ping': 'ping'})
    except:
      pass

    if code == 203:
      logging.info("%sWatcher detected a connection " % RUN_PREFIX)
      generate_reports()
    else:
       # Seconds for timeout
      seconds = 3**try_count
      
      # Exceeded max timeout
      if seconds >= 3600:
        exit()

      try_count += 1
      logging.info("%sWatcher did not detect a connection, retrying in %d seconds" % (RUN_PREFIX,seconds))

      # Sleep
      sleep(seconds)     

if __name__ == "__main__":
  run()
